import sqlite3
from update_labels import *
from add_tutos import *
from sql_request import *
from flask import Flask, request, render_template, render_template_string, jsonify

class CustomFlask(Flask):
    jinja_options = Flask.jinja_options.copy()
    jinja_options.update(dict(
        block_start_string='DD',
        block_end_string='DDE',
        variable_start_string='[[',
        variable_end_string=']]',
        comment_start_string='RF',
        comment_end_string='BG',
        ))

app = CustomFlask(__name__)
app.config['DEBUG'] = True

text_component = {"project_name": "LabelsTower",
    "labels_title_fr": "Étiquettes",
    "labels_title_ang": "Labels",
    "result_title_fr": "Tutoriels",
    "result_title_ang": "Tutorials"}

@app.route('/')
def home():
    return search_tuto()

@app.route('/get_datas/<string:name>', defaults={'id': None}, methods=['POST', 'GET'])
@app.route('/get_datas/<string:name>/<path:id>')
def get_datas(name, id):
    '''Return labels table or alias table for Ajax.
    '''
    if (name == 'labels'):
        return (jsonify(send_sql_request('SELECT * FROM labels')))
    if (name == 'all_labels'):
        return (jsonify(send_sql_request("SELECT id_label, labels_languages.name, father_id FROM labels_languages INNER JOIN labels ON labels.id = labels_languages.id_label  WHERE  labels_languages.country_code IN ('"+id+"',  'U')")))
    elif (name == 'main_labels'):
        return (jsonify(send_sql_request("SELECT * FROM (SELECT id_label, labels_languages.name, father_id FROM labels_languages INNER JOIN labels ON labels.id = labels_languages.id_label WHERE labels_languages.country_code IN ('"+id+"',  'U')) WHERE father_id IS NULL")))
    elif (name == 'labels_lang_id'):
        return (jsonify(send_sql_request('SELECT id FROM labels WHERE name IN ("Français", "Anglais")')))
    elif (name == 'alias'):
        return (jsonify(send_sql_request('SELECT * from alias')))
    elif (name == 'link'):
        return (jsonify(send_sql_request('SELECT id_element FROM elements_labels WHERE id_label = '+id)))
    elif (name == 'labels_links'):
        return (jsonify(send_sql_request('SELECT id_element FROM elements_labels WHERE id_label IN ('+id+')')))
    elif (name == 'labels_links_unselect'):
        return (jsonify(send_sql_request('SELECT id_element FROM elements_labels WHERE id_label NOT IN ('+id+')')))
    elif (name == 'title_tuto'): #anciennement tuto
        return (jsonify(send_sql_request("SELECT title FROM elements WHERE link = '"+id+"'")))
    elif (name == 'nb_labels'):
        return (jsonify(send_sql_request('SELECT COUNT (*) FROM labels')))
    elif (name == 'all_tutos'):
        return (jsonify(send_sql_request('SELECT * FROM elements ORDER BY title')))
    elif (name == 'tutos'):
        return (jsonify(send_sql_request('SELECT * FROM elements WHERE id IN ('+id+') ORDER BY title')))
    elif (name == 'id_labels_tutos'):
        return (jsonify(send_sql_request('SELECT DISTINCT id_label FROM elements_labels WHERE id_element IN ('+request.form['data']+')')))
    elif (name == 'labels_of_a_link'):
        return (jsonify(send_sql_request("SELECT id_label FROM elements_labels WHERE id_element = '"+id+"'")))
    elif (name == 'all_tutos_labels'):
        return (jsonify(send_sql_request("SELECT * FROM elements_labels")))
    elif (name == 'labels_of_a_link_in'):
        return (jsonify(send_sql_request('SELECT id_label FROM elements_labels WHERE id_element IN ('+id+')')))
    elif (name == 'tutos_language'):
        return (jsonify(send_sql_request('SELECT id_element from elements_labels WHERE id_label = (SELECT id FROM labels WHERE name = "'+id+'")')))
    elif (name == 'tuto_family'):
        ''' Select all links that have the same group of labels.
        '''
        labels_tab = id.split(',')
        sql_request = ''
        n = 0
        nb_labels = len(labels_tab) 
        for id_lab in labels_tab:
            if (n == nb_labels - 1):
                sql_request += 'SELECT id_element FROM elements_labels WHERE id_label IN ('+id_lab+')'
            else:
                sql_request += 'SELECT id_element FROM elements_labels WHERE id_label IN ('+id_lab+') INTERSECT '
            n += 1
        return (jsonify(send_sql_request(sql_request)))
    return 'Error'

@app.route('/manage_labels', methods=['GET', 'POST'])
def manage_labels():
    '''For manage all labels and alias register in database.
    (Add new labels and alias actually)
    '''
    if request.method == 'GET':
        res = send_sql_request('SELECT * FROM labels')
        return render_template('manage_labels.html', labels=res)
    else:
        update_labels(request.form['alias_input'])
        return render_template('manage_labels.html', labels=request.form['alias_input'], success="Succès de l'envoie")

@app.route('/browse')
def search_tuto():
    return render_template('browse.html', txt=text_component)

@app.route('/add_new_tuto', methods=['GET', 'POST'])
def add_new_tuto():
    '''Add new tuto in database.
    '''
    if request.method == 'GET':
        return render_template('add_new_tuto.html')
    else:
        res = add_tuto(request.form['title'], request.form['link'], request.form['labels'], request.form['prescriber'])
        return render_template('add_new_tuto.html', status=res)

@app.route('/test')
def test():
    cursor = connection.cursor()
    cursor.execute("SELECT * FROM labels")
    res = cursor.fetchall()
    connection.commit()
    print(res)
    return ("OK")

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=80)