'''Take user input and send labels and alias in database if they don't exist.
'''

import sqlite3
from sql_request import *
conn = sqlite3.connect('app/bdd_for_labels.db', check_same_thread=False)
c = conn.cursor()

def get_file_content(filename):
    '''For tests. Read file that contain user input.
    '''
    my_file = open(filename, "r")
    content = my_file.read()
    content = content.split('\n')
    my_file.close()
    return (content)

def put_labels_in_db(content, labels_table):
    '''Send all labels contained in content if it doesn't exist
    in labels table (in DB). If label line contain alias, just the
    first alias is send. executemany() requiere 2D array. It's why
    label is put in intermediate array 'inter'.
    '''
    labels_name = []
    for line in content:
        inter = []
        line = line.replace("\t", '')
        line = line.split(',')
        inter.append(line[0])
        if (is_in_labels_table(line[0], labels_table) != 1):
            labels_name.append(inter)
    c.executemany("INSERT INTO labels(name) VALUES(?)", labels_name)
    conn.commit()

def get_labels_table():
    '''Retrieve all the content in 'labels' from DB.
    '''
    c.execute('SELECT * FROM labels')
    all_table = c.fetchall()
    return (all_table)

def get_tab_alias_name():
    '''Select all alias in DB
    '''
    c.execute('SELECT name FROM alias')
    all_table = c.fetchall()
    return (all_table)

def is_in_labels_table(name, labels_table):
    '''Return 1 if name is in 'labels' table or 0 if not.
    '''
    name = name.lower()
    for label in labels_table:
        inter = label[1]
        inter = inter.lower()
        if (inter == name):
            return (1)
    return (0)

def is_an_alias(name, tab_alias_name):
    '''Return 1 if name is in 'alias' table or 0 if not.
    '''
    name = name.lower()
    for alias in tab_alias_name:
        inter = alias[0]
        inter = inter.lower()
        if (inter == name):
            return (1)
    return (0)

def is_alias_or_label(name, labels_table, tab_alias_name):
    '''Return 1 if name is a label or and alias.
    '''
    if (is_in_labels_table(name, labels_table) == 1):
        return (1)
    if (is_an_alias(name, tab_alias_name) == 1):
        return (1)
    return (0)


def get_last_element(tab):
    '''Retrive the last element from the pile 'tab' because pop()
    delete the last element when is it return.
    '''
    for case in tab:
        result = case
    return (case)

def get_father_id(father_name, labels_table):
    '''Return the ID of an label stock in 'labels' table.
    This ID will be send in 'father' column of the child.
    '''
    for label in labels_table:
        if (label[1] == father_name):
            return (label[0])
    return (-1)

def update_alias(content, labels_table, tab_alias_name):
    '''Check the user input and add alias in alias tab if it's not
    a label or an alias.
    '''
    label_alias = []
    for line in content:
        line = line.replace("\t", '')
        line = line.replace(", ", ',')
        line = line.split(',')
        label = line[0]
        line.remove(line[0])
        for alias in line:
            inter = []
            if (is_alias_or_label(alias, labels_table, tab_alias_name) != 1):
                inter.append(get_father_id(label, labels_table))
                inter.append(alias)
                label_alias.append(inter)
    c.executemany("INSERT INTO alias(id_label, name) VALUES(?, ?)", label_alias)
    conn.commit()

def update_father_id_in_db(content, labels_table):
    '''Get the user input and with the number of '\t' behind label
    dermine if the label is a father or a child.
    If it's a child, the id of his father is send in his
    'father_id' column.
    '''
    father_pile = ['no_father']
    father_name = ''
    prece = ''
    for line in content:
        line_alias = line.split(',')
        label = line_alias[0]
        if (line.count('\t') > prece.count('\t')):
            father_pile.append(prece.replace("\t", ''))
        if (line.count('\t') < prece.count('\t')):
            nb_tour = prece.count('\t') - line.count('\t')
            for i in range(0, nb_tour):
                father_pile.pop()
        father_name = get_last_element(father_pile)
        father_id = get_father_id(father_name, labels_table)
        if (father_id != -1):
            c.execute('UPDATE labels SET father_id = ? WHERE name = ?', (father_id, label.replace("\t", '')))
            conn.commit()
        prece = label

def update_labels(content):
    '''Get the user input and add new lables in 'labels' table of DB.
    '''
    #content = get_file_content("labels")
    content = content.split('\r\n')
    labels_table = get_labels_table()
    tab_alias_name = get_tab_alias_name()
    put_labels_in_db(content, labels_table)
    labels_table = get_labels_table()
    print (labels_table)
    update_father_id_in_db(content, labels_table)
    update_alias(content, labels_table, tab_alias_name)
