import sqlite3
import string
import sys
import os
from random import random, choice, randint

def send_sql_request(request, fetchall = 1, many = 0, values = []):
    '''Execute sql request on bdd_for_labels database.
    If second parameter is 0 return one line of the response
    else return all lines.
    '''
    global connection
    cursor = connection.cursor()
    if (many == 0):
        cursor.execute(request)
    else:
        cursor.executemany(request, values)
    connection.commit()
    if (fetchall == 1):
        res = cursor.fetchall()
    else:
        res = cursor.fetchone()
    return (res)

def create_database():
    req = [
        "CREATE TABLE IF NOT EXISTS tutos (title VARCHAR(300),link VARCHAR(500) NOT NULL,date DEFAULT CURRENT_TIMESTAMP,prescriber VARCHAR(100),CONSTRAINT pk_tutos PRIMARY KEY (link));"
        ,"CREATE TABLE IF NOT EXISTS labels (id INTEGER PRIMARY KEY AUTOINCREMENT,name VARCHAR(100) NOT NULL,father_id UNISGNED MEDIUM INT DEFAULT NULL);"
        ,"CREATE TABLE IF NOT EXISTS labels_languages (id_label INTEGER,country_code VARCHAR(3),name VARCHAR(100),CONSTRAINT pk_labels_languages PRIMARY KEY(id_label, country_code));"
        ,"CREATE TABLE IF NOT EXISTS tutos_labels (link VARCHAR(200) NOT NULL,id_label UNSIGNED MEDIUM INT,CONSTRAINT pk_tutos_labels PRIMARY KEY(link, id_label));"
    ]
    for request in req:
        send_sql_request(request)
    #les executer une par une

def random_generator(size=6):
    return (''.join(choice(string.ascii_uppercase + string.digits) for _ in range(size)))

def create_tutos(nb_tutos):
    name_tutos = []
    values1 = []
    for i in range (nb_tutos):
        values1.append([random_generator(), "http://"+random_generator(9)])
    request = "INSERT INTO tutos (title, link) VALUES (?, ?)"
    send_sql_request(request, many=1, values=values1)

def create_labels(nb_labels):
    values1 = []
    for i in range(nb_labels):
        father = randint(1, nb_labels)
        while (father == i):
            father = randint(1, nb_labels)
        values1.append([random_generator(7), father])
    request = "INSERT INTO labels (name, father_id) VALUES (?, ?)"
    send_sql_request(request, many=1, values=values1)

def link_tutos_labels(nb_tutos, nb_labels):
    all_links = send_sql_request("SELECT link FROM tutos")
    request = "INSERT INTO tutos_labels (link, id_label) VALUES (?, ?)"
    values1 = []
    for i in range (nb_tutos):
        nb_local_labels = randint(1, 5)
        all_id_tab = []
        for j in range(nb_local_labels):
            id_label = randint(1, nb_labels)
            while (id_label in all_id_tab):
                id_label = randint(1, nb_labels)
            values1.append([all_links[i][0], id_label])
            all_id_tab.append(id_label)
        # print (str(i)+"/"+str(nb_tutos))
    send_sql_request(request, many=1, values=values1)

def add_labels_language(nb_labels):
    labels = send_sql_request("SELECT id, name FROM labels")
    values1 = []
    for i in range(nb_labels):
        inter = [str(labels[i][0]), 'U', labels[i][1]]
        values1.append(inter)
    request = "INSERT INTO labels_languages (id_label, country_code, name) VALUES (?, ?, ?)"
    send_sql_request(request, many=1, values=values1)

def create_all_db(db_name, nb_tutos, nb_labels):
    create_database()
    create_tutos(nb_tutos)
    create_labels(nb_labels)
    add_labels_language(nb_labels)
    link_tutos_labels(nb_tutos, nb_labels)

if (len(sys.argv) == 2):
    print("exec [db_name] [nb_tutos] [nb_labels]") if (sys.argv[1] == "-h") else print("bad parameters")
elif (len(sys.argv) == 4):
    db_name = sys.argv[1]
    nb_tutos = int(sys.argv[2])
    nb_labels = int(sys.argv[3])
    connection = sqlite3.connect(db_name, check_same_thread=False)
    create_all_db(db_name, nb_tutos, nb_labels)
    print ("Succès")
else:
    print ("bad parameters, use \"create_random_db.py [db_name] [nb_tutos] [nb_labels]\"")

